@@ -10,17 +10,17 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import io
+import json
 import logging
 import random
 
 import fixtures
-import six
 
 from openstackclient import shell
 from oslotest import base
 from placement.tests.functional.fixtures import capture
 from placement.tests.functional.fixtures import placement
-import simplejson as json
 
 
 # A list of logger names that will be reset to a log level
@@ -52,7 +52,7 @@ class CommandException(Exception):
 
 
 class BaseTestCase(base.BaseTestCase):
-    VERSION = None
+    VERSION = '1.0'
 
     def setUp(self):
         super(BaseTestCase, self).setUp()
@@ -93,8 +93,8 @@ class BaseTestCase(base.BaseTestCase):
 
         # Context manager here instead of setUp because we only want
         # output trapping around the run().
-        self.output = six.StringIO()
-        self.error = six.StringIO()
+        self.output = io.StringIO()
+        self.error = io.StringIO()
         stdout_fix = fixtures.MonkeyPatch('sys.stdout', self.output)
         stderr_fix = fixtures.MonkeyPatch('sys.stderr', self.error)
         with stdout_fix, stderr_fix:
@@ -159,7 +159,7 @@ class BaseTestCase(base.BaseTestCase):
             self.fail('Command does not fail as required (%s)' % signature)
         except CommandException as e:
             self.assertIn(
-                message, six.text_type(e),
+                message, str(e),
                 'Command "%s" fails with different message' % e.cmd)
 
     def resource_provider_create(self,
@@ -178,7 +178,7 @@ class BaseTestCase(base.BaseTestCase):
                 self.resource_provider_delete(res['uuid'])
             except CommandException as exc:
                 # may have already been deleted by a test case
-                err_message = six.text_type(exc).lower()
+                err_message = str(exc).lower()
                 if 'no resource provider' not in err_message:
                     raise
         self.addCleanup(cleanup)
@@ -200,7 +200,8 @@ class BaseTestCase(base.BaseTestCase):
 
     def resource_provider_list(self, uuid=None, name=None,
                                aggregate_uuids=None, resources=None,
-                               in_tree=None, required=None, forbidden=None):
+                               in_tree=None, required=None, forbidden=None,
+                               member_of=None, may_print_to_stderr=False):
         to_exec = 'resource provider list'
         if uuid:
             to_exec += ' --uuid ' + uuid
@@ -217,21 +218,25 @@ class BaseTestCase(base.BaseTestCase):
             to_exec += ' ' + ' '.join('--required %s' % t for t in required)
         if forbidden:
             to_exec += ' ' + ' '.join('--forbidden %s' % f for f in forbidden)
+        if member_of:
+            to_exec += ' ' + ' '.join(
+                ['--member-of %s' % m for m in member_of])
 
-        return self.openstack(to_exec, use_json=True)
+        return self.openstack(
+            to_exec, use_json=True, may_print_to_stderr=may_print_to_stderr)
 
     def resource_provider_delete(self, uuid):
         return self.openstack('resource provider delete ' + uuid)
 
-    def resource_allocation_show(self, consumer_uuid):
-        return self.openstack(
-            'resource provider allocation show ' + consumer_uuid,
-            use_json=True
-        )
+    def resource_allocation_show(self, consumer_uuid, columns=()):
+        cmd = 'resource provider allocation show ' + consumer_uuid
+        cmd += ' '.join(' --column %s' % c for c in columns)
+        return self.openstack(cmd, use_json=True)
 
     def resource_allocation_set(self, consumer_uuid, allocations,
                                 project_id=None, user_id=None,
-                                use_json=True, may_print_to_stderr=False):
+                                consumer_type=None, use_json=True,
+                                may_print_to_stderr=False):
         cmd = 'resource provider allocation set {allocs} {uuid}'.format(
             uuid=consumer_uuid,
             allocs=' '.join('--allocation {}'.format(a) for a in allocations)
@@ -240,34 +245,40 @@ class BaseTestCase(base.BaseTestCase):
             cmd += ' --project-id %s' % project_id
         if user_id:
             cmd += ' --user-id %s' % user_id
-        result = self.openstack(cmd, use_json=use_json,
-                                may_print_to_stderr=may_print_to_stderr)
+        if consumer_type:
+            cmd += ' --consumer-type %s' % consumer_type
+        result = self.openstack(
+            cmd, use_json=use_json, may_print_to_stderr=may_print_to_stderr)
 
         def cleanup(uuid):
             try:
                 self.openstack('resource provider allocation delete ' + uuid)
             except CommandException as exc:
                 # may have already been deleted by a test case
-                if 'not found' in six.text_type(exc).lower():
+                if 'not found' in str(exc).lower():
                     pass
         self.addCleanup(cleanup, consumer_uuid)
 
         return result
 
-    def resource_allocation_unset(self, consumer_uuid, provider=None,
-                                  use_json=True):
+    def resource_allocation_unset(
+        self, consumer_uuid, provider=None, resource_class=None, use_json=True,
+        columns=(),
+    ):
+        cmd = 'resource provider allocation unset %s' % consumer_uuid
+        if resource_class:
+            cmd += ' ' + ' '.join(
+                '--resource-class %s' % rc for rc in resource_class)
         if provider:
             # --provider can be specified multiple times so if we only get
             # a single string value convert to a list.
-            if isinstance(provider, six.string_types):
+            if isinstance(provider, str):
                 provider = [provider]
-            cmd = 'resource provider allocation unset %s %s' % (
-                ' '.join('--provider %s' %
-                         rp_uuid for rp_uuid in provider),
-                consumer_uuid
-            )
-        else:
-            cmd = 'resource provider allocation unset %s' % consumer_uuid
+            cmd += ' ' + ' '.join(
+                '--provider %s' % rp_uuid for rp_uuid in provider)
+
+        cmd += ' '.join(' --column %s' % c for c in columns)
+
         result = self.openstack(cmd, use_json=use_json)
 
         def cleanup(uuid):
@@ -275,7 +286,7 @@ class BaseTestCase(base.BaseTestCase):
                 self.openstack('resource provider allocation delete ' + uuid)
             except CommandException as exc:
                 # may have already been deleted by a test case
-                if 'not found' in six.text_type(exc).lower():
+                if 'not found' in str(exc).lower():
                     pass
         self.addCleanup(cleanup, consumer_uuid)
 
@@ -285,15 +296,28 @@ class BaseTestCase(base.BaseTestCase):
         cmd = 'resource provider allocation delete ' + consumer_uuid
         return self.openstack(cmd)
 
-    def resource_inventory_show(self, uuid, resource_class):
-        cmd = 'resource provider inventory show {uuid} {rc}'.format(
-            uuid=uuid, rc=resource_class
+    def resource_inventory_show(
+        self, uuid, resource_class, *, include_used=False,
+    ):
+        resource = self.openstack(
+            f'resource provider inventory show {uuid} {resource_class}',
+            use_json=True,
         )
-        return self.openstack(cmd, use_json=True)
+        if not include_used:
+            del resource['used']
 
-    def resource_inventory_list(self, uuid):
-        return self.openstack('resource provider inventory list ' + uuid,
-                              use_json=True)
+        return resource
+
+    def resource_inventory_list(self, uuid, *, include_used=False):
+        resources = self.openstack(
+            f'resource provider inventory list {uuid}',
+            use_json=True,
+        )
+        if not include_used:
+            for resource in resources:
+                del resource['used']
+
+        return resources
 
     def resource_inventory_delete(self, uuid, resource_class=None):
         cmd = 'resource provider inventory delete {uuid}'.format(uuid=uuid)
@@ -382,7 +406,7 @@ class BaseTestCase(base.BaseTestCase):
                 self.trait_delete(name)
             except CommandException as exc:
                 # may have already been deleted by a test case
-                err_message = six.text_type(exc).lower()
+                err_message = str(exc).lower()
                 if 'http 404' not in err_message:
                     raise
         self.addCleanup(cleanup)
@@ -404,18 +428,45 @@ class BaseTestCase(base.BaseTestCase):
         cmd = 'resource provider trait delete %s ' % uuid
         self.openstack(cmd)
 
-    def allocation_candidate_list(self, resources, required=None,
+    def allocation_candidate_list(self, resources=None, required=None,
                                   forbidden=None, limit=None,
-                                  aggregate_uuids=None):
-        cmd = 'allocation candidate list ' + ' '.join(
-            '--resource %s' % resource for resource in resources)
-        if required is not None:
-            cmd += ''.join([' --required %s' % t for t in required])
-        if forbidden:
-            cmd += ' ' + ' '.join('--forbidden %s' % f for f in forbidden)
+                                  aggregate_uuids=None, member_of=None,
+                                  may_print_to_stderr=False):
+        cmd = 'allocation candidate list '
+        cmd += self._allocation_candidates_option(
+            resources, required, forbidden, aggregate_uuids, member_of)
         if limit is not None:
             cmd += ' --limit %d' % limit
-        if aggregate_uuids:
-            cmd += ' ' + ' '.join(
-                   '--aggregate-uuid %s' % a for a in aggregate_uuids)
+        return self.openstack(
+            cmd, use_json=True, may_print_to_stderr=may_print_to_stderr)
+
+    def allocation_candidate_granular(self, groups, group_policy=None,
+                                      limit=None):
+        cmd = 'allocation candidate list '
+        for suffix, req_group in groups.items():
+            if suffix:
+                cmd += ' --group %s' % suffix
+            cmd += self._allocation_candidates_option(**req_group)
+            if limit is not None:
+                cmd += ' --limit %d' % limit
+            if group_policy is not None:
+                cmd += ' --group-policy %s' % group_policy
         return self.openstack(cmd, use_json=True)
+
+    def _allocation_candidates_option(self, resources=None, required=None,
+                                      forbidden=None, aggregate_uuids=None,
+                                      member_of=None):
+        opt = ''
+        if resources:
+            opt += ' ' + ' '.join(
+                '--resource %s' % resource for resource in resources)
+        if required is not None:
+            opt += ''.join([' --required %s' % t for t in required])
+        if forbidden:
+            opt += ' ' + ' '.join('--forbidden %s' % f for f in forbidden)
+        if aggregate_uuids:
+            opt += ' ' + ' '.join(
+                '--aggregate-uuid %s' % a for a in aggregate_uuids)
+        if member_of:
+            opt += ' ' + ' '.join(['--member-of %s' % m for m in member_of])
+        return opt
